{% extends "_base.html" %}

{% block home %} 
{% endblock %}

{% block body %}

<div class="jumbotron urubu">
    <div class="container text-center">
        <div class="row">
            <div class="col-md-8 col-md-offset-2">
                <h1>{{this.title}}</h1>
                <h2>{{this.tagline}}</h2>
             </div>
        </div>
    </div>
</div>

<div class="container">
<main>
{% set news = site.reflinks['/news'].content[0] %}
<p class="lead text-center">{{news.date|dateformat}}: <a href="news">{{news.title}}</a></p>
{{this.body}}
</main>
</div>

{% include 'footer.html' %}

{% endblock %}
