{% extends "_base.html" %}

{% block body %}

{% block jumbotron %}
<div class="jumbotron">
  <div class="container">
    <div class="row">
       <div class="col-md-7">
         <h1>{{this.title}}</h1>
         <p>{{this.abstract}}</p>
    </div>
  </div>
 </div>
</div>
{% endblock %}

<div class="container">
  <div class="row">

    <div class="col-md-7" role="main">
    <main>
    {% for item in this.content %}
      <h4><a href="{{item.url}}">{{item.title}}</a></h4>
        <p>{{item.abstract}}</p>
    {% endfor %}
    </main>
    </div>

 </div>
</div>

{% endblock %}
