    <div class="footer">
        <div style="margin: 5px">
        {% include 'sharing.html' %}
        </div>
        <p> Content licensed under the
           <a href="{{site.reflinks.content_license.url}}">{{site.reflinks.content_license.title}}</a>.
        </p> 
        <p>Website powered by <a href="http://urubu.jandecaluwe.com">Urubu</a>.</p>
    </div>


